package com.javacodegeeks.sangria

import sangria.ast.Document
import sangria.execution.deferred.DeferredResolver
import sangria.execution.{ErrorWithResolver, Executor, QueryAnalysisError}
import sangria.parser.{QueryParser, SyntaxError}
import sangria.parser.DeliveryScheme.Try
import sangria.marshalling.circe._
import akka.actor.ActorSystem
import akka.http.scaladsl.Http
import akka.http.scaladsl.model.StatusCodes._
import akka.http.scaladsl.server.Directives._
import akka.http.scaladsl.model.MediaTypes._
import akka.http.scaladsl.server._
import akka.stream.ActorMaterializer
import de.heikoseeberger.akkahttpcirce.ErrorAccumulatingCirceSupport._
import io.circe._
import io.circe.optics.JsonPath._
import io.circe.parser._

import scala.util.control.NonFatal
import scala.util.{Failure, Success}
import sangria.slowlog.SlowLog

object Server extends App {
  private[this] val libraryService = new LibraryServiceImpl

  implicit val system = ActorSystem("sangria-graphql")
  implicit val materializer = ActorMaterializer()

  import system.dispatcher

  
  def executeGraphQL(query: Document, operationName: Option[String], variables: Json) =
    complete(Executor.execute(SchemaDefinition.LibrarySchema, query, libraryService,
      variables = if (variables.isNull) Json.obj() else variables,
      operationName = operationName)
        .map(OK → _)
        .recover {
          case error: QueryAnalysisError ⇒ BadRequest → error.resolveError
          case error: ErrorWithResolver ⇒ InternalServerError → error.resolveError
        })

  val route: Route =
    get {
      getFromResource("graphiql.html")
    } ~
    path("graphql") {
      post {
        parameters('query.?, 'operationName.?, 'variables.?) { (queryParam, operationNameParam, variablesParam) ⇒
          entity(as[Json]) { body ⇒
            val query = queryParam orElse root.query.string.getOption(body)
            val operationName = operationNameParam orElse root.operationName.string.getOption(body)
            val variablesStr = variablesParam orElse root.variables.string.getOption(body)

            query.map(QueryParser.parse(_)) match {
              case Some(Success(ast)) ⇒
                variablesStr.map(parse) match {
                  case Some(Left(error)) ⇒ complete(BadRequest)
                  case Some(Right(json)) ⇒ executeGraphQL(ast, operationName, json)
                  case None ⇒ executeGraphQL(ast, operationName, root.variables.json.getOption(body) getOrElse Json.obj())
                }
              case Some(Failure(error)) ⇒ complete(BadRequest)
              case None ⇒ complete(BadRequest)
            }
          }
        }
      }
    } 

  Http().bindAndHandle(route, "localhost", 8686)
}
